package org.eclipse.viatra.integration.uml.derivedfeatures;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.uml2.uml.Transition;
import org.eclipse.uml2.uml.Vertex;
import org.eclipse.viatra.integration.uml.derivedfeatures.VertexIncomingMatch;
import org.eclipse.viatra.integration.uml.derivedfeatures.util.VertexIncomingQuerySpecification;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.integration.uml.derivedfeatures.vertexIncoming pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link VertexIncomingMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * (if extendedSignature-{@literal >}isEmpty() then Set{} else extendedSignature.parameter-{@literal >}asSet() endif)
 * 
 * // Can't compile OCL to VIATRA Query because of java.lang.NullPointerException
 * //{@literal @}Surrogate(feature = "inheritedParameter")
 * //{@literal @}QueryExplorer(checked = false)
 * //pattern redefinableTemplateSignatureInheritedParameter(source: RedefinableTemplateSignature, target: TemplateParameter) {}
 * 
 * 
 * (region-{@literal >}size () {@literal >} 1)
 * 
 * // Can't compile OCL to VIATRA Query because of java.lang.UnsupportedOperationException: size
 * //{@literal @}Surrogate(feature = "isOrthogonal")
 * //{@literal @}QueryExplorer(checked = false)
 * //pattern stateIsOrthogonal(source: State, target: Boolean) {}
 * 
 * 
 * ((region-{@literal >}isEmpty()) and not isSubmachineState())
 * 
 * // Can't compile OCL to VIATRA Query because of java.lang.UnsupportedOperationException: isSubmachineState
 * //{@literal @}Surrogate(feature = "isSimple")
 * //{@literal @}QueryExplorer(checked = false)
 * //pattern stateIsSimple(source: State, target: Boolean) {}
 * 
 * 
 * (submachine {@literal <}{@literal >} null)
 * 
 * // Can't compile OCL to VIATRA Query because of java.lang.IllegalArgumentException: Unhandled parameter types: [null]
 * //{@literal @}Surrogate(feature = "isSubmachineState")
 * //{@literal @}QueryExplorer(checked = false)
 * //pattern stateIsSubmachineState(source: State, target: Boolean) {}
 * 
 * 
 * (self.containingProfile())
 * 
 * // Can't compile OCL to VIATRA Query because of java.lang.UnsupportedOperationException: containingProfile
 * //{@literal @}Surrogate(feature = "profile")
 * //{@literal @}QueryExplorer(checked = false)
 * //pattern stereotypeProfile(source: Stereotype, target: Profile) {}
 * 
 * 
 * (Transition.allInstances()-{@literal >}select(target=self))
 * 
 * {@literal @}Surrogate(feature = "incoming")
 * {@literal @}QueryExplorer(checked = false)
 * pattern vertexIncoming(
 *     self : Vertex,
 *     temp1 : Transition
 * ) {
 *     Transition.target(temp1, vertex);
 *     vertex == self;
 *     Transition(transition);
 *     temp1 == transition;
 * }
 * </pre></code>
 * 
 * @see VertexIncomingMatch
 * @see VertexIncomingProcessor
 * @see VertexIncomingQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class VertexIncomingMatcher extends BaseMatcher<VertexIncomingMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static VertexIncomingMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    VertexIncomingMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new VertexIncomingMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SELF = 0;
  
  private final static int POSITION_TEMP1 = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(VertexIncomingMatcher.class);
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  private VertexIncomingMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @return matches represented as a VertexIncomingMatch object.
   * 
   */
  public Collection<VertexIncomingMatch> getAllMatches(final Vertex pSelf, final Transition pTemp1) {
    return rawGetAllMatches(new Object[]{pSelf, pTemp1});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @return a match represented as a VertexIncomingMatch object, or null if no match is found.
   * 
   */
  public VertexIncomingMatch getOneArbitraryMatch(final Vertex pSelf, final Transition pTemp1) {
    return rawGetOneArbitraryMatch(new Object[]{pSelf, pTemp1});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Vertex pSelf, final Transition pTemp1) {
    return rawHasMatch(new Object[]{pSelf, pTemp1});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Vertex pSelf, final Transition pTemp1) {
    return rawCountMatches(new Object[]{pSelf, pTemp1});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Vertex pSelf, final Transition pTemp1, final IMatchProcessor<? super VertexIncomingMatch> processor) {
    rawForEachMatch(new Object[]{pSelf, pTemp1}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Vertex pSelf, final Transition pTemp1, final IMatchProcessor<? super VertexIncomingMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSelf, pTemp1}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pTemp1 the fixed value of pattern parameter temp1, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public VertexIncomingMatch newMatch(final Vertex pSelf, final Transition pTemp1) {
    return VertexIncomingMatch.newMatch(pSelf, pTemp1);
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Vertex> rawAccumulateAllValuesOfself(final Object[] parameters) {
    Set<Vertex> results = new HashSet<Vertex>();
    rawAccumulateAllValues(POSITION_SELF, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Vertex> getAllValuesOfself() {
    return rawAccumulateAllValuesOfself(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Vertex> getAllValuesOfself(final VertexIncomingMatch partialMatch) {
    return rawAccumulateAllValuesOfself(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Vertex> getAllValuesOfself(final Transition pTemp1) {
    return rawAccumulateAllValuesOfself(new Object[]{
    null, 
    pTemp1
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for temp1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Transition> rawAccumulateAllValuesOftemp1(final Object[] parameters) {
    Set<Transition> results = new HashSet<Transition>();
    rawAccumulateAllValues(POSITION_TEMP1, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for temp1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Transition> getAllValuesOftemp1() {
    return rawAccumulateAllValuesOftemp1(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for temp1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Transition> getAllValuesOftemp1(final VertexIncomingMatch partialMatch) {
    return rawAccumulateAllValuesOftemp1(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for temp1.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Transition> getAllValuesOftemp1(final Vertex pSelf) {
    return rawAccumulateAllValuesOftemp1(new Object[]{
    pSelf, 
    null
    });
  }
  
  @Override
  protected VertexIncomingMatch tupleToMatch(final Tuple t) {
    try {
    	return VertexIncomingMatch.newMatch((Vertex) t.get(POSITION_SELF), (Transition) t.get(POSITION_TEMP1));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected VertexIncomingMatch arrayToMatch(final Object[] match) {
    try {
    	return VertexIncomingMatch.newMatch((Vertex) match[POSITION_SELF], (Transition) match[POSITION_TEMP1]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected VertexIncomingMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return VertexIncomingMatch.newMutableMatch((Vertex) match[POSITION_SELF], (Transition) match[POSITION_TEMP1]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<VertexIncomingMatcher> querySpecification() throws ViatraQueryException {
    return VertexIncomingQuerySpecification.instance();
  }
}
