package org.eclipse.viatra.query.tooling.ui.retevis;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.rete.recipes.Mask;
import org.eclipse.viatra.query.runtime.rete.recipes.ProjectionIndexerRecipe;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.query.tooling.ui.retevis.ProjectionIndexerRecipeMatch;
import org.eclipse.viatra.query.tooling.ui.retevis.util.ProjectionIndexerRecipeQuerySpecification;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.query.tooling.ui.retevis.projectionIndexerRecipe pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link ProjectionIndexerRecipeMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * {@literal @}Item(item = recipe, label = "IX: $mask.sourceIndices$/$mask.sourceArity$")
 * {@literal @}Format(textColor = "#000000", color = "#FFFFFF")
 * {@literal @}QueryExplorer(checked = false)
 * pattern projectionIndexerRecipe(recipe : ProjectionIndexerRecipe, mask : Mask) {
 * 	ProjectionIndexerRecipe(recipe);
 * 	ProjectionIndexerRecipe.mask(recipe, mask);
 * }
 * </pre></code>
 * 
 * @see ProjectionIndexerRecipeMatch
 * @see ProjectionIndexerRecipeProcessor
 * @see ProjectionIndexerRecipeQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class ProjectionIndexerRecipeMatcher extends BaseMatcher<ProjectionIndexerRecipeMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static ProjectionIndexerRecipeMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    ProjectionIndexerRecipeMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new ProjectionIndexerRecipeMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_RECIPE = 0;
  
  private final static int POSITION_MASK = 1;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(ProjectionIndexerRecipeMatcher.class);
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  private ProjectionIndexerRecipeMatcher(final ViatraQueryEngine engine) throws ViatraQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return matches represented as a ProjectionIndexerRecipeMatch object.
   * 
   */
  public Collection<ProjectionIndexerRecipeMatch> getAllMatches(final ProjectionIndexerRecipe pRecipe, final Mask pMask) {
    return rawGetAllMatches(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return a match represented as a ProjectionIndexerRecipeMatch object, or null if no match is found.
   * 
   */
  public ProjectionIndexerRecipeMatch getOneArbitraryMatch(final ProjectionIndexerRecipe pRecipe, final Mask pMask) {
    return rawGetOneArbitraryMatch(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final ProjectionIndexerRecipe pRecipe, final Mask pMask) {
    return rawHasMatch(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final ProjectionIndexerRecipe pRecipe, final Mask pMask) {
    return rawCountMatches(new Object[]{pRecipe, pMask});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final ProjectionIndexerRecipe pRecipe, final Mask pMask, final IMatchProcessor<? super ProjectionIndexerRecipeMatch> processor) {
    rawForEachMatch(new Object[]{pRecipe, pMask}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final ProjectionIndexerRecipe pRecipe, final Mask pMask, final IMatchProcessor<? super ProjectionIndexerRecipeMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pRecipe, pMask}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pRecipe the fixed value of pattern parameter recipe, or null if not bound.
   * @param pMask the fixed value of pattern parameter mask, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public ProjectionIndexerRecipeMatch newMatch(final ProjectionIndexerRecipe pRecipe, final Mask pMask) {
    return ProjectionIndexerRecipeMatch.newMatch(pRecipe, pMask);
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<ProjectionIndexerRecipe> rawAccumulateAllValuesOfrecipe(final Object[] parameters) {
    Set<ProjectionIndexerRecipe> results = new HashSet<ProjectionIndexerRecipe>();
    rawAccumulateAllValues(POSITION_RECIPE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfrecipe() {
    return rawAccumulateAllValuesOfrecipe(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfrecipe(final ProjectionIndexerRecipeMatch partialMatch) {
    return rawAccumulateAllValuesOfrecipe(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for recipe.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProjectionIndexerRecipe> getAllValuesOfrecipe(final Mask pMask) {
    return rawAccumulateAllValuesOfrecipe(new Object[]{
    null, 
    pMask
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Mask> rawAccumulateAllValuesOfmask(final Object[] parameters) {
    Set<Mask> results = new HashSet<Mask>();
    rawAccumulateAllValues(POSITION_MASK, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask() {
    return rawAccumulateAllValuesOfmask(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask(final ProjectionIndexerRecipeMatch partialMatch) {
    return rawAccumulateAllValuesOfmask(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for mask.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Mask> getAllValuesOfmask(final ProjectionIndexerRecipe pRecipe) {
    return rawAccumulateAllValuesOfmask(new Object[]{
    pRecipe, 
    null
    });
  }
  
  @Override
  protected ProjectionIndexerRecipeMatch tupleToMatch(final Tuple t) {
    try {
    	return ProjectionIndexerRecipeMatch.newMatch((ProjectionIndexerRecipe) t.get(POSITION_RECIPE), (Mask) t.get(POSITION_MASK));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected ProjectionIndexerRecipeMatch arrayToMatch(final Object[] match) {
    try {
    	return ProjectionIndexerRecipeMatch.newMatch((ProjectionIndexerRecipe) match[POSITION_RECIPE], (Mask) match[POSITION_MASK]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected ProjectionIndexerRecipeMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return ProjectionIndexerRecipeMatch.newMutableMatch((ProjectionIndexerRecipe) match[POSITION_RECIPE], (Mask) match[POSITION_MASK]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<ProjectionIndexerRecipeMatcher> querySpecification() throws ViatraQueryException {
    return ProjectionIndexerRecipeQuerySpecification.instance();
  }
}
