/**
 * Copyright (c) 2004-2015, Peter Lunk, Zoltan Ujhelyi and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.transformation.debug.configuration;

import com.google.common.collect.Lists;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.viatra.transformation.debug.TransformationDebugAdapter;
import org.eclipse.viatra.transformation.debug.TransformationDebugListener;
import org.eclipse.viatra.transformation.debug.breakpoints.ITransformationBreakpoint;
import org.eclipse.viatra.transformation.debug.controller.IDebugController;
import org.eclipse.viatra.transformation.debug.controller.impl.ConsoleDebugger;
import org.eclipse.viatra.transformation.evm.api.adapter.IAdapterConfiguration;
import org.eclipse.viatra.transformation.evm.api.adapter.IEVMAdapter;
import org.eclipse.viatra.transformation.evm.api.adapter.IEVMListener;

/**
 * Configuration class that defines the debugger.
 * 
 * @author Peter Lunk
 */
@SuppressWarnings("all")
public class TransformationDebuggerConfiguration implements IAdapterConfiguration {
  private List<IEVMAdapter> adapters;
  
  private List<IEVMListener> listeners;
  
  public TransformationDebuggerConfiguration(final IDebugController usedController, final ITransformationBreakpoint... breakpoints) {
    ArrayList<IEVMAdapter> _newArrayList = Lists.<IEVMAdapter>newArrayList();
    this.adapters = _newArrayList;
    ArrayList<IEVMListener> _newArrayList_1 = Lists.<IEVMListener>newArrayList();
    this.listeners = _newArrayList_1;
    TransformationDebugListener _transformationDebugListener = new TransformationDebugListener(usedController);
    this.listeners.add(_transformationDebugListener);
    TransformationDebugAdapter _transformationDebugAdapter = new TransformationDebugAdapter(usedController, breakpoints);
    this.adapters.add(_transformationDebugAdapter);
  }
  
  public TransformationDebuggerConfiguration(final ITransformationBreakpoint... breakpoints) {
    final ConsoleDebugger usedUI = new ConsoleDebugger();
    ArrayList<IEVMAdapter> _newArrayList = Lists.<IEVMAdapter>newArrayList();
    this.adapters = _newArrayList;
    ArrayList<IEVMListener> _newArrayList_1 = Lists.<IEVMListener>newArrayList();
    this.listeners = _newArrayList_1;
    TransformationDebugListener _transformationDebugListener = new TransformationDebugListener(usedUI);
    this.listeners.add(_transformationDebugListener);
    TransformationDebugAdapter _transformationDebugAdapter = new TransformationDebugAdapter(usedUI, breakpoints);
    this.adapters.add(_transformationDebugAdapter);
  }
  
  @Override
  public List<IEVMAdapter> getAdapters() {
    return this.adapters;
  }
  
  @Override
  public List<IEVMListener> getListeners() {
    return this.listeners;
  }
}
