/**
 * Copyright (c) 2004-2015, Peter Lunk, Zoltan Ujhelyi and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.transformation.debug.controller.impl;

import com.google.common.collect.Maps;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;
import java.util.Set;
import org.eclipse.viatra.transformation.debug.DebuggerActions;
import org.eclipse.viatra.transformation.debug.controller.IDebugController;
import org.eclipse.viatra.transformation.evm.api.Activation;
import org.eclipse.viatra.transformation.evm.api.RuleInstance;
import org.eclipse.viatra.transformation.evm.api.RuleSpecification;
import org.eclipse.xtext.xbase.lib.Exceptions;

/**
 * Default debugger UI implementation that uses the Eclipse console.
 * 
 * @author Peter Lunk
 */
@SuppressWarnings("all")
public class ConsoleDebugger implements IDebugController {
  private Map<Integer, Activation<?>> activationMap = Maps.<Integer, Activation<?>>newHashMap();
  
  @Override
  public void displayConflictingActivations(final Set<Activation<?>> activations) {
    System.out.println("Please order the following CONFLICTING rule activations: ");
    int i = 0;
    for (final Activation<?> act : activations) {
      {
        String _plus = (Integer.valueOf(i) + ". - ");
        String _string = act.toString();
        String _plus_1 = (_plus + _string);
        System.out.println(_plus_1);
        this.activationMap.put(Integer.valueOf(i), act);
        i++;
      }
    }
  }
  
  @Override
  public void displayTransformationContext(final Activation<?> act) {
    String _string = act.toString();
    String _plus = ("activation: " + _string);
    System.out.println(_plus);
    RuleInstance<?> _instance = act.getInstance();
    RuleSpecification<?> _specification = _instance.getSpecification();
    String _string_1 = _specification.toString();
    String _plus_1 = ("ruleSpec: " + _string_1);
    System.out.println(_plus_1);
    boolean _isEnabled = act.isEnabled();
    String _plus_2 = ("enabled: " + Boolean.valueOf(_isEnabled));
    System.out.println(_plus_2);
  }
  
  @Override
  public DebuggerActions getDebuggerAction() {
    DebuggerActions _xblockexpression = null;
    {
      System.out.println(" Type \"step\" to step or \"continue\" to continue execution");
      DebuggerActions action = null;
      boolean running = true;
      while (running) {
        try {
          InputStreamReader _inputStreamReader = new InputStreamReader(System.in);
          final BufferedReader bufferedReader = new BufferedReader(_inputStreamReader);
          final String input = bufferedReader.readLine();
          String _lowerCase = input.toLowerCase();
          boolean _equals = _lowerCase.equals("step");
          if (_equals) {
            action = DebuggerActions.Step;
            running = false;
          } else {
            String _lowerCase_1 = input.toLowerCase();
            boolean _equals_1 = _lowerCase_1.equals("continue");
            if (_equals_1) {
              action = DebuggerActions.Continue;
              running = false;
            } else {
              System.out.println("Inappropriate input");
            }
          }
        } catch (final Throwable _t) {
          if (_t instanceof IOException) {
            final IOException e = (IOException)_t;
            System.out.println("Inappropriate input");
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
      }
      _xblockexpression = action;
    }
    return _xblockexpression;
  }
  
  @Override
  public Activation<?> getSelectedActivation() {
    Activation<?> temp = null;
    System.out.println("Please insert the numeric ID of the next activation.");
    try {
      InputStreamReader _inputStreamReader = new InputStreamReader(System.in);
      final BufferedReader bufferedReader = new BufferedReader(_inputStreamReader);
      String _readLine = bufferedReader.readLine();
      final int sel = Integer.parseInt(_readLine);
      Activation<?> _get = this.activationMap.get(Integer.valueOf(sel));
      temp = _get;
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
        System.out.println("Inappropriate input");
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    this.activationMap.clear();
    return temp;
  }
}
