/**
 * Copyright (c) 2004-2015, Peter Lunk, Zoltan Ujhelyi and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Peter Lunk - initial API and implementation
 */
package org.eclipse.viatra.transformation.tracer.tracecoder;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.viatra.transformation.evm.api.Activation;
import org.eclipse.viatra.transformation.evm.api.adapter.AbstractEVMListener;
import org.eclipse.viatra.transformation.tracer.activationcoder.IActivationCoder;
import org.eclipse.viatra.transformation.tracer.activationcoder.impl.DefaultActivationCoder;
import org.eclipse.viatra.transformation.tracer.tracemodelserializer.ITraceModelSerializer;
import org.eclipse.viatra.transformation.tracer.tracemodelserializer.impl.DefaultTraceModelSerializer;
import org.eclipse.viatra.transformation.tracer.transformationtrace.ActivationTrace;
import org.eclipse.viatra.transformation.tracer.transformationtrace.TransformationTrace;
import org.eclipse.viatra.transformation.tracer.transformationtrace.TransformationtraceFactory;
import org.eclipse.xtext.xbase.lib.Extension;

/**
 * Adapter implementation that creates transformation traces based on the ongoing transformation.
 * 
 *  @author Peter Lunk
 */
@SuppressWarnings("all")
public class TraceCoder extends AbstractEVMListener {
  @Extension
  private TransformationtraceFactory factory = TransformationtraceFactory.eINSTANCE;
  
  @Extension
  private IActivationCoder activationCoder;
  
  private TransformationTrace trace;
  
  private ITraceModelSerializer serializer;
  
  public TraceCoder(final IActivationCoder activationCoder, final ITraceModelSerializer serializer) {
    this.activationCoder = activationCoder;
    this.serializer = serializer;
    TransformationTrace _createTransformationTrace = this.factory.createTransformationTrace();
    this.trace = _createTransformationTrace;
  }
  
  public TraceCoder(final IActivationCoder activationCoder, final URI location) {
    this.activationCoder = activationCoder;
    DefaultTraceModelSerializer _defaultTraceModelSerializer = new DefaultTraceModelSerializer(location);
    this.serializer = _defaultTraceModelSerializer;
    TransformationTrace _createTransformationTrace = this.factory.createTransformationTrace();
    this.trace = _createTransformationTrace;
  }
  
  public TraceCoder(final URI location) {
    DefaultActivationCoder _defaultActivationCoder = new DefaultActivationCoder();
    this.activationCoder = _defaultActivationCoder;
    DefaultTraceModelSerializer _defaultTraceModelSerializer = new DefaultTraceModelSerializer(location);
    this.serializer = _defaultTraceModelSerializer;
    TransformationTrace _createTransformationTrace = this.factory.createTransformationTrace();
    this.trace = _createTransformationTrace;
  }
  
  @Override
  public void beforeFiring(final Activation<?> activation) {
    EList<ActivationTrace> _activationTraces = this.trace.getActivationTraces();
    ActivationTrace _createActivationCode = this.activationCoder.createActivationCode(activation);
    _activationTraces.add(_createActivationCode);
  }
  
  @Override
  public void endTransaction(final String transactionID) {
    this.serializer.serializeTraceModel(this.trace);
  }
}
